/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

#include "nxp_kernel.h"
#include "i18n.h"

NxpElementInt *nxp_named_args_get_int(GHashTable *hash, const char *key,
				      NxpEvalContext *context, GError **error) {
    GError *err = NULL;
    gpointer ptr = NULL;
    NxpElement *el1, *el2;

    if (!hash) return NULL;
    
    ptr = g_hash_table_lookup(hash, key);

    if (ptr) {
	el1 = NXP_ELEMENT(ptr);
	el2 = nxp_element_eval(el1, context, &err);
	if (!el2 || err) {
	    g_propagate_error(error, err);
	    return NULL;
	}

	if (NXP_ELEMENT_IS_INT(el2)) {
	    return NXP_ELEMENT_INT(el2);
	}
	if (NXP_ELEMENT_IS_REAL(el2)) {
	    el1 = nxp_element_int_new(nxp_element_r_get_value_as_int(NXP_ELEMENT_R(el2)));
	    NXP_ELEMENT_UNREF(el2);
	    return NXP_ELEMENT_INT(el1);
	}

	NXP_ELEMENT_UNREF(el2);
	g_set_error(error, nxp_error_domain, NXP_ERROR_TYPE_CHECK, _("Named argument with incorrect data type"));
	return NULL;
    } else {
	/* not set the error... not found if not _really_ an error */
	return NULL;
    }
}

NxpElementReal *nxp_named_args_get_real(GHashTable *hash, const char *key,
					NxpEvalContext *context, GError **error) {
    GError *err = NULL;
    gpointer ptr = NULL;
    NxpElement *el1, *el2;
    
    if (!hash) return NULL;

    ptr = g_hash_table_lookup(hash, key);

    if (ptr) {
	el1 = NXP_ELEMENT(ptr);
	el2 = nxp_element_eval(el1, context, &err);
	if (!el2 || err) {
	    g_propagate_error(error, err);
	    return NULL;
	}

	if (NXP_ELEMENT_IS_REAL(el2)) {
	    return NXP_ELEMENT_REAL(el2);
	}
	if (NXP_ELEMENT_IS_INT(el2)) {
	    el1 = nxp_element_real_new(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(el2)));
	    NXP_ELEMENT_UNREF(el2);
	    return NXP_ELEMENT_REAL(el1);	    
	}

	NXP_ELEMENT_UNREF(el2);
	g_set_error(error, nxp_error_domain, NXP_ERROR_TYPE_CHECK, _("Named argument with incorrect data type"));
	return NULL;
    } else {
	/* not set the error... not found if not _really_ an error */
	return NULL;
    }
}
