/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

#include "nxp_kernel.h"
#include "parser/parser.h"
#include "parser/files.h"
#include "i18n.h"

#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
#include <libintl.h>
#endif


NumexpDebugFlags _numexp_debug_flags;

char* nxp_kernel_version(void) {
    return VERSION;
}

void nxp_kernel_init(void)
{
    static gboolean kernel_initialized = FALSE;
    GError *error = NULL;
    static GDebugKey debug_keys[] = {
	{ "parser",    NUMEXP_DEBUG_PARSER },
	{ "elements",  NUMEXP_DEBUG_ELEMENTS },
    };
    const char *env_string;

    if (kernel_initialized) return;
    kernel_initialized = TRUE;

    g_message(__FUNCTION__);
    
    _numexp_debug_flags = NUMEXP_DEBUG_NONE;
    env_string = g_getenv("NUMEXP_DEBUG");
    if (env_string)
	_numexp_debug_flags |=
	    g_parse_debug_string(env_string,
				 debug_keys,
				 G_N_ELEMENTS(debug_keys));
    if (_numexp_debug_flags)
	g_message("_numexp_debug_flags = %i", _numexp_debug_flags);

    /* init thread-safe glib, unless it has already been initialized. */
    if (!g_thread_supported())
	g_thread_init(NULL);

    nxp_elements_init();

    // This needs to be called before the nxp_types_init!!
    nxp_typecasts_init();
    nxp_types_init(&error);
    nxp_prototypes_init();
    // We shouldn't do this, this way...
    if (error) g_error("%s", error->message);

    exp_parser_init();

#ifdef ENABLE_NLS
    bindtextdomain(GETTEXT_PACKAGE, NUMEXP_CORE_LOCALEDIR);
#    ifdef HAVE_BIND_TEXTDOMAIN_CODESET
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
#    endif
#endif  

}


void nxp_kernel_load_rc(const char *rcfile, NxpEvalContext *context) 
{
    GError *error = NULL;
    
    if (rcfile) {
	NxpElement *el;
	el = nxp_files_load(rcfile, context, &error);
	if (error) {
	    g_message("%s", error->message);
	    g_error_free(error);
	} else {
	    if (el) NXP_ELEMENT_UNREF(el);
	    else g_message("Error executing startup file '%s'", rcfile);
	}
    } else {
	NxpElement *el;
	el = nxp_files_rc_load(context, &error);

	if (error) {
	    g_message("%s", error->message);
	    g_error_free(error);
	} else {
	    if (el) NXP_ELEMENT_UNREF(el);
	    else g_message("Error evaluating .numexprc");
	}
    }
}

