/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

/* Vector Any relation */

#include <math.h>

#include "vector.h"
#include "tree.h"
#include "container.h"
#include <element.h>

#include "ourlibc.h"

#include "op_bindings.h"
#include "types_misc.h"
#include "message.h"
#include "errors.h"
#include "i18n.h"


#define BIND_SYM(op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_VECTOR, \
                                                   NXP_ELEMENT_TYPE_ANY,        \
						   map_v_a_eval, map_v_a_simplify,              \
						   NXP_OP_BIND_SYMMETRIC,  \
						   NULL);

#define BIND_V_A(op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_VECTOR, \
                                                        NXP_ELEMENT_TYPE_ANY,        \
							map_v_a_eval, map_v_a_simplify,              \
							0,  \
							NULL);

#define BIND_A_V(op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_ANY, \
                                                        NXP_ELEMENT_TYPE_VECTOR,        \
							map_a_v_eval, map_a_v_simplify,              \
							0,  \
							NULL);

#define BIND_FULL(f1,f2,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_VECTOR, \
                                                        NXP_ELEMENT_TYPE_ANY,        \
							f1, f2,              \
							0,  \
							NULL);

#define BIND_FULL_V_INT(f1,f2,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_VECTOR, \
                                                        NXP_ELEMENT_TYPE_INT,        \
							f1, f2,              \
							0,  \
							NULL);
#define BIND_SYM_FULL(f1,f2,op)    nxp_bind_operator_function(op,                    \
							      NXP_ELEMENT_TYPE_VECTOR,    \
							      NXP_ELEMENT_TYPE_ANY,       \
							      f1, f2,                \
							      NXP_OP_BIND_SYMMETRIC, \
							      NULL);



static NxpElement* or(NxpOp op, NxpElement *a, NxpElement *b,
		      NxpEvalContext *context, GError **error) {
    return nxp_element_int_new((NXP_ELEMENT_VECTOR(a)->size)?1:nxp_element_is_true(b, context, error));
}

static NxpElement* and(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((NXP_ELEMENT_VECTOR(a)->size)?nxp_element_is_true(b, context, error):0);
}


static NxpElement* map_v_a_simplify(NxpOp op, NxpElement *a, NxpElement *b,
					NxpEvalContext *context, GError **error)
{
    GError *err = NULL;
    NxpElementVector *result;
    gint i;

    g_return_val_if_fail(NXP_ELEMENT_IS_VECTOR(a), NULL);
    g_return_val_if_fail(b, NULL);

    result = NXP_ELEMENT_VECTOR(nxp_element_vector_new(NXP_ELEMENT_VECTOR(a)->size,
						       NULL));
    for (i = 0; i < NXP_ELEMENT_VECTOR(a)->size; i++)
    {
	_element_vector_index(result, i) = nxp_element_tree_simplify_op(
				op,
				_element_vector_index(NXP_ELEMENT_VECTOR(a), i),
				b, context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    NXP_ELEMENT_UNREF(result);
	    return NULL;
	}
    }
    return NXP_ELEMENT(result);
}

static NxpElement* map_v_a_eval(NxpOp op, NxpElement *a, NxpElement *b,
				NxpEvalContext *context, GError **error)
{
    NxpElementVector *result;
    GError *err = NULL;
    gint i;

    g_return_val_if_fail(NXP_ELEMENT_IS_VECTOR(a), NULL);
    g_return_val_if_fail(b, NULL);

    result = NXP_ELEMENT_VECTOR(nxp_element_vector_new(NXP_ELEMENT_VECTOR(a)->size,
						       NULL));
    for (i = 0; i < NXP_ELEMENT_VECTOR(a)->size; i++)
    {
	_element_vector_index(result, i) = nxp_element_tree_eval_op(
				op,
				_element_vector_index(NXP_ELEMENT_VECTOR(a), i),
				b, context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    NXP_ELEMENT_UNREF(result);
	    return NULL;
	}
    }
    return NXP_ELEMENT(result);
}

static NxpElement* map_a_v_simplify(NxpOp op, NxpElement *a, NxpElement *b,
				    NxpEvalContext *context, GError **error)
{
    NxpElementVector *result;
    gint i;
    GError *err = NULL;

    g_return_val_if_fail(a, NULL);
    g_return_val_if_fail(NXP_ELEMENT_IS_VECTOR(b), NULL);

    result = NXP_ELEMENT_VECTOR(nxp_element_vector_new(NXP_ELEMENT_VECTOR(b)->size,
						       NULL));
    for (i = 0; i < NXP_ELEMENT_VECTOR(b)->size; i++)
    {
	_element_vector_index(result, i) = nxp_element_tree_simplify_op(
				op, a,
				_element_vector_index(NXP_ELEMENT_VECTOR(b), i),
				context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    NXP_ELEMENT_UNREF(result);
	    return NULL;
	}
    }
    return NXP_ELEMENT(result);
}

static NxpElement* map_a_v_eval(NxpOp op, NxpElement *a, NxpElement *b,
				NxpEvalContext *context, GError **error)
{
    NxpElementVector *result;
    gint i;
    GError *err = NULL;

    g_return_val_if_fail(a, NULL);
    g_return_val_if_fail(NXP_ELEMENT_IS_VECTOR(b), NULL);

    result = NXP_ELEMENT_VECTOR(nxp_element_vector_new(NXP_ELEMENT_VECTOR(b)->size,
						       NULL));
    for (i = 0; i < NXP_ELEMENT_VECTOR(b)->size; i++)
    {
	_element_vector_index(result, i) = nxp_element_tree_eval_op(
				op, a,
				_element_vector_index(NXP_ELEMENT_VECTOR(b), i),
				context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    NXP_ELEMENT_UNREF(result);
	    return NULL;
	}
    }
    return NXP_ELEMENT(result);
}

/*  _element_vector_mul_shl:
 *  @v: a vector
 *  @n: an element
 *  @sh, an integer
 *
 *  @Shift the vector of sh rows to the left(sh must be a non negative number),
 *  fill it with zeros on the right and multiply it by n.
 *
 * Return value: the new vector.
 */
NxpElement* _element_vector_mul_shl(NxpElement *v, NxpElement *n, int sh, NxpEvalContext *context, GError **error)
{
    NxpElementVector *result;
    gint i;
    GError *err = NULL;

    g_return_val_if_fail(NXP_ELEMENT_IS_VECTOR(v), NULL);
    g_return_val_if_fail(n, NULL);
    g_return_val_if_fail(sh >= 0, NULL);

    result = NXP_ELEMENT_VECTOR(nxp_element_vector_new(NXP_ELEMENT_VECTOR(v)->size + sh,
						       NULL));
    for (i = 0; i < NXP_ELEMENT_VECTOR(v)->size; i++)
    {
	_element_vector_index(result, i + sh) = nxp_element_tree_simplify_op(
				NXP_OP_STAR,
				_element_vector_index(NXP_ELEMENT_VECTOR(v), i),
				n, context, &err);
	if (err) {
	    NXP_ELEMENT_UNREF(result);
	    g_propagate_error(error, err);
	    return NULL;
	}
    }
    for (i = 0; i < sh; i++)
	_element_vector_index(result, i) = nxp_element_int_new(0);

    return NXP_ELEMENT(result);
}

void nxp_element_vector_any_operator_init(void)
{
    BIND_SYM(NXP_OP_PLUS);
    BIND_V_A(NXP_OP_MINUS);
    BIND_A_V(NXP_OP_MINUS);

    BIND_SYM(NXP_OP_STAR);
    BIND_V_A(NXP_OP_SLASH);
    BIND_V_A(NXP_OP_PERCENT);
    BIND_A_V(NXP_OP_PERCENT);
    BIND_A_V(NXP_OP_SLASH);
    BIND_V_A(NXP_OP_HAT);
    BIND_A_V(NXP_OP_HAT);

    // NXP_OP_MINUS_LEFT --> handled on other files

    //    NXP_OP_DOT3
    //    NXP_OP_PLUS2_LEFT
    //    NXP_OP_MINUS2_LEFT
    //    NXP_OP_PLUS2_RIGHT
    //    NXP_OP_MINUS2_RIGHT
    //    NXP_OP_GT
    //    NXP_OP_GT_EQ
    //    NXP_OP_LT
    //    NXP_OP_LT_EQ
    //    NXP_OP_EQ2
    //    NXP_OP_EXCL_EQ

    BIND_SYM_FULL(and, and, NXP_OP_SLASH_BSLASH);
    BIND_SYM_FULL(or, or, NXP_OP_BSLASH_SLASH);
    //    NXP_OP_TILDE_LEFT
    //    NXP_OP_EXCL_RIGHT
    //    NXP_OP_EQ
    //    NXP_OP_COLON_EQ
}
