/* -*- c -*- */
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include "nxp_kernel.h"
#include "types/nxp_complex.h"
#include "gsl.h"

#define NXP2GSL(a) nxp_gsl_complex_from_nxp_complex(a)
#define GSL2NXP(a) nxp_complex_from_gsl_complex(a)

static NxpElement *
nxp_gsl_complexes_from_polar(NxpElementReal * r, NxpElementReal * theta,
			     ETC)
{
    GSL_RETURN(nxp_complex_from_gsl_complex
	       (gsl_complex_polar(nxp_element_r_get_value_as_double(r), nxp_element_r_get_value_as_double(theta))));
}

static NxpElement *nxp_gsl_complex_logabs(NxpElementComplex *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_complex_logabs(NXP2GSL(arg1));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_complex_sqrt_real(NxpElementReal *arg1, ETC)
{
	gsl_complex ret;
	ret = gsl_complex_sqrt_real(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(GSL2NXP(ret));
}

static NxpElement *nxp_gsl_complex_pow(NxpElementComplex *arg1 ,NxpElementComplex *arg2, ETC)
{
	gsl_complex ret;
	ret = gsl_complex_pow(NXP2GSL(arg1) ,NXP2GSL(arg2));
	GSL_RETURN(GSL2NXP(ret));
}

static NxpElement *nxp_gsl_complex_pow_real(NxpElementComplex *arg1 ,NxpElementReal *arg2, ETC)
{
	gsl_complex ret;
	ret = gsl_complex_pow_real(NXP2GSL(arg1) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(GSL2NXP(ret));
}

static NxpElement *nxp_gsl_complex_log10(NxpElementComplex *arg1, ETC)
{
	gsl_complex ret;
	ret = gsl_complex_log10(NXP2GSL(arg1));
	GSL_RETURN(GSL2NXP(ret));
}

static NxpElement *nxp_gsl_complex_log_b(NxpElementComplex *arg1 ,NxpElementComplex *arg2, ETC)
{
	gsl_complex ret;
	ret = gsl_complex_log_b(NXP2GSL(arg1) ,NXP2GSL(arg2));
	GSL_RETURN(GSL2NXP(ret));
}


static NxpElement *
nxp_gsl_complexes_op_pow(NxpOp op, NxpElement * a, NxpElement * b, SIMPLE_ETC)
{
    GSL_RETURN(GSL2NXP(gsl_complex_pow(NXP2GSL(NXP_ELEMENT_COMPLEX(a)),
				       NXP2GSL(NXP_ELEMENT_COMPLEX(b)))));
}

static NxpElement *
nxp_gsl_complexes_op_pow_real(NxpOp op, NxpElement * a, NxpElement * b,
			      SIMPLE_ETC)
{
    GSL_RETURN(GSL2NXP
	       (gsl_complex_pow_real
		(NXP2GSL(NXP_ELEMENT_COMPLEX(a)),
		 nxp_element_r_get_value_as_double(NXP_ELEMENT_R(b)))));
}


void
nxp_gsl_complexes_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd, "gsl");
id = NXP_ID_2("gsl", "complexFromPolar");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_complexes_from_polar, NULL, nxp_gsl_complexes_from_polar, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "lnabs");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX, nxp_gsl_complex_logabs, NULL, nxp_gsl_complex_logabs, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "sqrt");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_complex_sqrt_real, NULL, nxp_gsl_complex_sqrt_real, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "pow");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX_COMPLEX, nxp_gsl_complex_pow, NULL, nxp_gsl_complex_pow, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "pow");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX_REAL, nxp_gsl_complex_pow_real, NULL, nxp_gsl_complex_pow_real, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "log");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX, nxp_gsl_complex_log10, NULL, nxp_gsl_complex_log10, context, NULL); nxp_identifier_unref(id);
id = NXP_ID_2("gsl", "log");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX_COMPLEX, nxp_gsl_complex_log_b, NULL, nxp_gsl_complex_log_b, context, NULL); nxp_identifier_unref(id);

    /*
     * operators 
     */
    nxp_bind_operator_function(NXP_OP_HAT,
			       NXP_ELEMENT_TYPE_COMPLEX,
			       NXP_ELEMENT_TYPE_COMPLEX,
			       nxp_gsl_complexes_op_pow,
			       nxp_gsl_complexes_op_pow, 0, NULL);

    nxp_bind_operator_function(NXP_OP_HAT,
			       NXP_ELEMENT_TYPE_COMPLEX,
			       NXP_ELEMENT_TYPE_REAL,
			       nxp_gsl_complexes_op_pow_real,
			       nxp_gsl_complexes_op_pow_real, 0, NULL);
}
