/* -*- c -*- */
#include <gsl/gsl_integration.h>
#include "nxp_kernel.h"
#include "gsl.h"
#include "i18n.h"


typedef struct _eval_data {
    NxpEvalContext *context;
    NxpIdentifier *identifier;
} EvalData;

static double eval_f(double x, void *data) {
    GError *err = NULL;
    NxpElement *function;
    NxpIdentifier *id;
    NxpElement *answer;
    EvalData* ed = (EvalData*)data;
    double val = NAN;
    GSList *l;
    NxpElement *args;


    id = ed->identifier;
    l = g_slist_append(NULL, nxp_element_real_new(x));
    args = nxp_element_list_new_from_list(l);

    function = nxp_element_function_new(id, args, NULL);
    NXP_ELEMENT_UNREF(args);

    if (!function) return NAN;

    answer = nxp_element_eval(function, ed->context, &err);
    NXP_ELEMENT_UNREF(function);
    if (!answer || err) return NAN;

    if (NXP_ELEMENT_GET_TYPE(answer) == NXP_ELEMENT_TYPE_REAL)
	val = nxp_element_r_get_value_as_double(NXP_ELEMENT_R(answer));
    if (NXP_ELEMENT_GET_TYPE(answer) == NXP_ELEMENT_TYPE_INT)
	val = nxp_element_r_get_value_as_double(NXP_ELEMENT_R(answer));

    NXP_ELEMENT_UNREF(answer);
    return val;
}

static NxpElement * nxp_gsl_integration_qng(NxpElementVariable* f, NxpElementReal* a, NxpElementReal* b, NxpElementReal* epsabs, NxpElementReal* epsrel, ETC)
{
//    gsl_integration_workspace* w = gsl_integration_workspace_alloc(1000);
    double result, gsl_error;
    EvalData data;
    NxpIdentifier *func_id, *ufunc_id;
    NxpDefinition *func_def;
    double aa, bb, epsa, epsr;
    size_t neval;
    gsl_function function;

    func_id = f->ident;
    func_def = nxp_definition_get(func_id, context);

    if (!func_def) {
	ufunc_id = nxp_identifier_create_function_identifier(func_id, 1);
	func_def = nxp_definition_get(ufunc_id, context);
	nxp_identifier_unref(ufunc_id);
    }

    if (func_def && (func_def->type == NXP_DEF_USER_FUNC ||
		     func_def->type == NXP_DEF_BI_FUNC)) {
	data.identifier = nxp_identifier_ref(func_id);
	data.context = context;

	aa = nxp_element_r_get_value_as_double(a);
	bb = nxp_element_r_get_value_as_double(b);
	epsa = nxp_element_r_get_value_as_double(epsabs);
	epsr = nxp_element_r_get_value_as_double(epsrel);

	function.function = &eval_f;
	function.params   = &data;

	gsl_integration_qng(&function, aa, bb, epsa, epsr, &result, &gsl_error, &neval);
	GSL_RETURN(nxp_element_real_new(result));
    } else {
	g_set_error(error, nxp_error_domain, NXP_ERROR_UNDEFINED, 
		    _("Undefined function"));
	return NULL;
    }
}

static NxpElement * nxp_gsl_integration_qag(NxpElementVariable* f, NxpElementReal* a, NxpElementReal* b, NxpElementReal* epsabs, NxpElementReal* epsrel, ETC)
{
    gsl_integration_workspace* w = gsl_integration_workspace_alloc(1000);
    double result, gsl_error;
    EvalData data;
    NxpIdentifier *func_id, *ufunc_id;
    NxpDefinition *func_def;
    double aa, bb, epsa, epsr;
    size_t neval;
    NxpElement *retval;
    gsl_function function;
    int key = GSL_INTEG_GAUSS41; /* can be 15, 21, 31, 41, 51, 61 */
    func_id = f->ident;
    func_def = nxp_definition_get(func_id, context);

    if (!func_def) {
	ufunc_id = nxp_identifier_create_function_identifier(func_id, 1);
	func_def = nxp_definition_get(ufunc_id, context);
	nxp_identifier_unref(ufunc_id);
    }

    if (func_def && (func_def->type == NXP_DEF_USER_FUNC ||
		     func_def->type == NXP_DEF_BI_FUNC)) {
	data.identifier = nxp_identifier_ref(func_id);
	data.context = context;

	aa = nxp_element_r_get_value_as_double(a);
	bb = nxp_element_r_get_value_as_double(b);
	epsa = nxp_element_r_get_value_as_double(epsabs);
	epsr = nxp_element_r_get_value_as_double(epsrel);

	function.function = &eval_f;
	function.params   = &data;

	gsl_integration_qag(&function, aa, bb, epsa, epsr, 1000, key, w, &result, &gsl_error);
	retval = nxp_element_real_new(result);
    } else {
	g_set_error(error, nxp_error_domain, NXP_ERROR_UNDEFINED, 
		    _("Undefined function"));
	retval = NULL;
    }
    gsl_integration_workspace_free(w);
    GSL_RETURN(retval);
}

void
nxp_gsl_integration_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"integration");
id = nxp_identifier_new_with_full_path(3,"gsl","integration", "integQNG");
 nxp_bifunc_define(id, &nxp_prototype__VARIABLE_REAL_REAL_REAL_REAL, nxp_gsl_integration_qng, NULL, nxp_gsl_integration_qng, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","integration", "integQAG");
 nxp_bifunc_define(id, &nxp_prototype__VARIABLE_REAL_REAL_REAL_REAL, nxp_gsl_integration_qag, NULL, nxp_gsl_integration_qag, context, NULL); nxp_identifier_unref(id);
}
