/* -*- c -*- */
#include <gsl/gsl_rng.h>
#include <time.h>
#include "nxp_kernel.h"
#include "gsl.h"

static NxpElement *
nxp_gsl_rng_get(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el = nxp_element_real_new(gsl_rng_get(rng));
    GSL_RETURN(el);
}

static NxpElement *
nxp_gsl_rng_set(NxpElementInt * seed, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    gsl_rng_set(rng, nxp_element_r_get_value_as_int(seed));
    GSL_RETURN(nxp_element_void_new());
}

static NxpElement *
nxp_gsl_rng_uniform(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el = nxp_element_real_new(gsl_rng_uniform(rng));
    GSL_RETURN(el);
}

static NxpElement *
nxp_gsl_rng_uniform_int(NxpElementInt * seed, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_rng_uniform_int(rng, nxp_element_r_get_value_as_int(seed)));
    GSL_RETURN(el);
}

static NxpElement *
nxp_gsl_rng_name(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el = nxp_element_string_new(gsl_rng_name(rng));
    GSL_RETURN(el);
}

static void
change(NxpEvalContext * context, const gsl_rng_type * T)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    gsl_rng_free(rng);
    rng = gsl_rng_alloc(T);
    nxp_eval_context_set_user_data(context, "RNG", rng);


}

static NxpElement *
nxp_gsl_rng_mt19937(ETC)
{
    change(context, gsl_rng_mt19937);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_taus(ETC)
{
    change(context, gsl_rng_taus);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_mrg(ETC)
{
    change(context, gsl_rng_mrg);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_cmrg(ETC)
{
    change(context, gsl_rng_cmrg);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_gfsr4(ETC)
{
    change(context, gsl_rng_gfsr4);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlux(ETC)
{
    change(context, gsl_rng_ranlux);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlux389(ETC)
{
    change(context, gsl_rng_ranlux389);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlxd1(ETC)
{
    change(context, gsl_rng_ranlxd1);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlxd2(ETC)
{
    change(context, gsl_rng_ranlxd2);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlxs0(ETC)
{
    change(context, gsl_rng_ranlxs0);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlxs1(ETC)
{
    change(context, gsl_rng_ranlxs1);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranlxs2(ETC)
{
    change(context, gsl_rng_ranlxs2);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_rand(ETC)
{
    change(context, gsl_rng_rand);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_random_bsd(ETC)
{
    change(context, gsl_rng_random_bsd);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_random_libc5(ETC)
{
    change(context, gsl_rng_random_libc5);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_random_glibc2(ETC)
{
    change(context, gsl_rng_random_glibc2);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_rand48(ETC)
{
    change(context, gsl_rng_rand48);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ran0(ETC)
{
    change(context, gsl_rng_ran0);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ran1(ETC)
{
    change(context, gsl_rng_ran1);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ran2(ETC)
{
    change(context, gsl_rng_ran2);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ran3(ETC)
{
    change(context, gsl_rng_ran3);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranf(ETC)
{
    change(context, gsl_rng_ranf);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_ranmar(ETC)
{
    change(context, gsl_rng_ranmar);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_r250(ETC)
{
    change(context, gsl_rng_r250);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_tt800(ETC)
{
    change(context, gsl_rng_tt800);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_vax(ETC)
{
    change(context, gsl_rng_vax);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_transputer(ETC)
{
    change(context, gsl_rng_transputer);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_randu(ETC)
{
    change(context, gsl_rng_randu);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_minstd(ETC)
{
    change(context, gsl_rng_minstd);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_uni(ETC)
{
    change(context, gsl_rng_uni);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_uni32(ETC)
{
    change(context, gsl_rng_uni32);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_slatec(ETC)
{
    change(context, gsl_rng_slatec);
    return nxp_element_void_new();
}

static NxpElement *
nxp_gsl_rng_zuf(ETC)
{
    change(context, gsl_rng_zuf);
    return nxp_element_void_new();
}

void
nxp_gsl_rng_init(NxpEvalContext * context)
{
    gsl_rng        *rng;
    NxpIdentifier  *id;

    /*
     * By default, a gsl_rng_taus (at least, at the moment) 
     */
    rng = gsl_rng_alloc(gsl_rng_taus);
    gsl_rng_set(rng, time(NULL));
    nxp_eval_context_set_user_data(context, "RNG", rng);

nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"RNG");
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "rand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_get, NULL, nxp_gsl_rng_get, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "seed");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_rng_set, NULL, nxp_gsl_rng_set, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "uniformRand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_uniform, NULL, nxp_gsl_rng_uniform, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "uniformRand");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_rng_uniform_int, NULL, nxp_gsl_rng_uniform_int, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "current");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_name, NULL, nxp_gsl_rng_name, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_mt19937");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_mt19937, NULL, nxp_gsl_rng_mt19937, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_taus");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_taus, NULL, nxp_gsl_rng_taus, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_mrg");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_mrg, NULL, nxp_gsl_rng_mrg, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_cmrg");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_cmrg, NULL, nxp_gsl_rng_cmrg, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_gfsr4");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_gfsr4, NULL, nxp_gsl_rng_gfsr4, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlux");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlux, NULL, nxp_gsl_rng_ranlux, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlux389");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlux389, NULL, nxp_gsl_rng_ranlux389, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlxd1");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlxd1, NULL, nxp_gsl_rng_ranlxd1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlxd2");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlxd2, NULL, nxp_gsl_rng_ranlxd2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlxs0");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlxs0, NULL, nxp_gsl_rng_ranlxs0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlxs1");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlxs1, NULL, nxp_gsl_rng_ranlxs1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranlxs2");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranlxs2, NULL, nxp_gsl_rng_ranlxs2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_stdRand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_rand, NULL, nxp_gsl_rng_rand, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_bsdRand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_random_bsd, NULL, nxp_gsl_rng_random_bsd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_libc5Rand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_random_libc5, NULL, nxp_gsl_rng_random_libc5, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_glibc2Rand");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_random_glibc2, NULL, nxp_gsl_rng_random_glibc2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_rand48");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_rand48, NULL, nxp_gsl_rng_rand48, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ran0");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ran0, NULL, nxp_gsl_rng_ran0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ran1");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ran1, NULL, nxp_gsl_rng_ran1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ran2");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ran2, NULL, nxp_gsl_rng_ran2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ran3");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ran3, NULL, nxp_gsl_rng_ran3, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranf");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranf, NULL, nxp_gsl_rng_ranf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_ranmar");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_ranmar, NULL, nxp_gsl_rng_ranmar, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_r250");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_r250, NULL, nxp_gsl_rng_r250, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_tt800");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_tt800, NULL, nxp_gsl_rng_tt800, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_vax");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_vax, NULL, nxp_gsl_rng_vax, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_transputer");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_transputer, NULL, nxp_gsl_rng_transputer, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_randu");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_randu, NULL, nxp_gsl_rng_randu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_minstd");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_minstd, NULL, nxp_gsl_rng_minstd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_uni");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_uni, NULL, nxp_gsl_rng_uni, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_uni32");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_uni32, NULL, nxp_gsl_rng_uni32, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_slatec");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_slatec, NULL, nxp_gsl_rng_slatec, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","RNG", "init_zuf");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_rng_zuf, NULL, nxp_gsl_rng_zuf, context, NULL); nxp_identifier_unref(id);
}
