/*-*- c -*- */
#include <gsl/gsl_sf_coulomb.h>
#include <gsl/gsl_errno.h>
#include "nxp_kernel.h"
#include "gsl.h"

/*
 * STATUS: DONE 
 */

static NxpElement *nxp_gsl_sf_hydrogenicR_1(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_hydrogenicR_1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_hydrogenicR(NxpElementInt *arg1 ,NxpElementInt *arg2 ,NxpElementReal *arg3 ,NxpElementReal *arg4, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_hydrogenicR(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg2), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg4)));
	GSL_RETURN(nxp_element_real_new(ret));
}


NxpElement     *
nxp_gsl_sf_coulomb_wave_FG_e(NxpElementReal * ETA,
			     NxpElementReal * X,
			     NxpElementReal * L_F, NxpElementInt * K, ETC)
{
    gsl_sf_result   F,
                    FP,
                    G,
                    GP;
    double          EXP_F,
                    EXP_G;
    NxpElement     *vec;

    gsl_sf_coulomb_wave_FG_e(nxp_element_r_get_value_as_double(ETA), nxp_element_r_get_value_as_double(X), nxp_element_r_get_value_as_double(L_F), nxp_element_r_get_value_as_int(K),
			     &F, &FP, &G, &GP, &EXP_F, &EXP_G);

    vec = nxp_element_vector_new(4, NULL);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 0) =
	nxp_element_real_new(F.val);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 1) =
	nxp_element_real_new(FP.val);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 2) =
	nxp_element_real_new(G.val);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 3) =
	nxp_element_real_new(GP.val);

    GSL_RETURN(vec);
}

NxpElement     *
nxp_gsl_sf_coulomb_wave_F_array(NxpElementReal * L_MIN,
				NxpElementInt * KMAX,
				NxpElementReal * ETA,
				NxpElementReal * X, ETC)
{
    double          F_EXPONENT;
    double         *buffer;
    int             i;
    NxpElement     *vec;

    buffer = g_new(double, nxp_element_r_get_value_as_int(KMAX) + 1);

    gsl_sf_coulomb_wave_F_array(nxp_element_r_get_value_as_double(L_MIN), nxp_element_r_get_value_as_int(KMAX), nxp_element_r_get_value_as_double(ETA),
				nxp_element_r_get_value_as_double(X), buffer, &F_EXPONENT);

    vec = nxp_element_vector_new(nxp_element_r_get_value_as_int(KMAX) + 1, NULL);
    for (i = 0; i <= nxp_element_r_get_value_as_int(KMAX); i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(buffer[i]);
    }
    g_free(buffer);

    GSL_RETURN(vec);
}


NxpElement     *
nxp_gsl_sf_coulomb_wave_FG_array(NxpElementReal * L_MIN,
				 NxpElementInt * KMAX,
				 NxpElementReal * ETA,
				 NxpElementReal * X, ETC)
{
    double         *fc_ARRAY,
                   *gc_ARRAY;
    double          F_EXPONENT,
                    G_EXPONENT;

    NxpElement     *v1,
                   *v2,
                   *vec;

    int             i,
                    size = nxp_element_r_get_value_as_int(KMAX) + 1;

    fc_ARRAY = g_new(double, size);
    gc_ARRAY = g_new(double, size);

    vec = NULL;

    if (GSL_SUCCESS ==
	gsl_sf_coulomb_wave_FG_array(nxp_element_r_get_value_as_double(L_MIN), nxp_element_r_get_value_as_int(KMAX), nxp_element_r_get_value_as_double(ETA),
				     nxp_element_r_get_value_as_double(X), fc_ARRAY, gc_ARRAY,
				     &F_EXPONENT, &G_EXPONENT)) {
	v1 = nxp_element_vector_new(size, NULL);
	v2 = nxp_element_vector_new(size, NULL);
	vec = nxp_element_vector_new(2, NULL);

	for (i = 0; i < size; i++) {
	    _element_vector_index(NXP_ELEMENT_VECTOR(v1), i) =
		nxp_element_real_new(fc_ARRAY[i]);
	    _element_vector_index(NXP_ELEMENT_VECTOR(v2), i) =
		nxp_element_real_new(gc_ARRAY[i]);
	}

	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 0) = v1;
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 1) = v2;
    }

    g_free(fc_ARRAY);
    g_free(gc_ARRAY);


    GSL_RETURN(vec);
    return NULL;
}

NxpElement     *
nxp_gsl_sf_coulomb_wave_FGp_array(NxpElementReal * L_MIN,
				  NxpElementInt * KMAX,
				  NxpElementReal * ETA,
				  NxpElementReal * X, ETC)
{
    double         *fc_ARRAY,
                   *gc_ARRAY;
    double         *fcp_ARRAY,
                   *gcp_ARRAY;
    double          F_EXPONENT,
                    G_EXPONENT;

    NxpElement     *vec = NULL;

    int             size = nxp_element_r_get_value_as_int(KMAX) + 1;

    fc_ARRAY = g_new(double, size);
    gc_ARRAY = g_new(double, size);
    gcp_ARRAY = g_new(double, size);
    fcp_ARRAY = g_new(double, size);

    if (GSL_SUCCESS ==
	gsl_sf_coulomb_wave_FGp_array(nxp_element_r_get_value_as_double(L_MIN), nxp_element_r_get_value_as_int(KMAX),
				      nxp_element_r_get_value_as_double(ETA), nxp_element_r_get_value_as_double(X), fc_ARRAY,
				      fcp_ARRAY, gc_ARRAY, gcp_ARRAY,
				      &F_EXPONENT, &G_EXPONENT)) {
	int             i;
	NxpElement     *v_fc,
	               *v_fcp,
	               *v_gc,
	               *v_gcp;

	v_fc = nxp_element_vector_new(size, NULL);
	v_gc = nxp_element_vector_new(size, NULL);
	v_fcp = nxp_element_vector_new(size, NULL);
	v_gcp = nxp_element_vector_new(size, NULL);

	for (i = 0; i < size; i++) {
	    _element_vector_index(NXP_ELEMENT_VECTOR(v_fc), i) =
		nxp_element_real_new(fc_ARRAY[i]);
	    _element_vector_index(NXP_ELEMENT_VECTOR(v_gc), i) =
		nxp_element_real_new(gc_ARRAY[i]);
	    _element_vector_index(NXP_ELEMENT_VECTOR(v_fcp), i) =
		nxp_element_real_new(fcp_ARRAY[i]);
	    _element_vector_index(NXP_ELEMENT_VECTOR(v_gcp), i) =
		nxp_element_real_new(gcp_ARRAY[i]);
	}

	vec = nxp_element_vector_new(4, NULL);
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 0) = v_fc;
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 1) = v_fcp;
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 2) = v_gc;
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), 3) = v_gcp;

    }

    g_free(fc_ARRAY);
    g_free(gc_ARRAY);
    g_free(fcp_ARRAY);
    g_free(gcp_ARRAY);

    GSL_RETURN(vec);
}

NxpElement     *
nxp_gsl_sf_coulomb_wave_sphF_array(NxpElementReal * L_MIN,
				   NxpElementInt * KMAX,
				   NxpElementReal * ETA,
				   NxpElementReal * X, ETC)
{
    double          F_EXPONENT;
    double         *buffer;
    int             i;
    NxpElement     *vec;

    buffer = g_new(double, nxp_element_r_get_value_as_int(KMAX) + 1);

    gsl_sf_coulomb_wave_sphF_array(nxp_element_r_get_value_as_double(L_MIN), nxp_element_r_get_value_as_int(KMAX), nxp_element_r_get_value_as_double(ETA),
				   nxp_element_r_get_value_as_double(X), buffer, &F_EXPONENT);

    vec = nxp_element_vector_new(nxp_element_r_get_value_as_int(KMAX) + 1, NULL);
    for (i = 0; i <= nxp_element_r_get_value_as_int(KMAX); i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(buffer[i]);
    }
    g_free(buffer);

    GSL_RETURN(vec);
}

NxpElement     *
nxp_gsl_sf_coulomb_CL_e(NxpElementReal * L, NxpElementReal * ETA, ETC)
{
    gsl_sf_result   RESULT;
    gsl_sf_coulomb_CL_e(nxp_element_r_get_value_as_double(L), nxp_element_r_get_value_as_double(ETA), &RESULT);
    GSL_RETURN(nxp_element_real_new(RESULT.val));

}

NxpElement     *
nxp_gsl_sf_coulomb_CL_array(NxpElementReal * LMIN,
			    NxpElementInt * KMAX,
			    NxpElementReal * ETA, ETC)
{
    int             i,
                    size = nxp_element_r_get_value_as_int(KMAX) + 1;
    double         *buffer = g_new(double, size);
    NxpElement     *vec;

    gsl_sf_coulomb_CL_array(nxp_element_r_get_value_as_double(LMIN), nxp_element_r_get_value_as_int(KMAX), nxp_element_r_get_value_as_double(ETA), buffer);
    vec = nxp_element_vector_new(size, NULL);

    for (i = 0; i < size; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(buffer[i]);
    }

    g_free(buffer);

    GSL_RETURN(vec);
}


void
nxp_gsl_sf_coulomb_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"coulomb");
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "hydrogenicR1");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_hydrogenicR_1, NULL, nxp_gsl_sf_hydrogenicR_1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "hydrogenicR");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL_REAL, nxp_gsl_sf_hydrogenicR, NULL, nxp_gsl_sf_hydrogenicR, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "waveFG");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL_INT, nxp_gsl_sf_coulomb_wave_FG_e, NULL, nxp_gsl_sf_coulomb_wave_FG_e, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "waveF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT_REAL_REAL, nxp_gsl_sf_coulomb_wave_F_array, NULL, nxp_gsl_sf_coulomb_wave_F_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "waveRangeFG");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT_REAL_REAL, nxp_gsl_sf_coulomb_wave_FG_array, NULL, nxp_gsl_sf_coulomb_wave_FG_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "waveRangeFGp");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT_REAL_REAL, nxp_gsl_sf_coulomb_wave_FGp_array, NULL, nxp_gsl_sf_coulomb_wave_FGp_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "waveRangeSph");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT_REAL_REAL, nxp_gsl_sf_coulomb_wave_sphF_array, NULL, nxp_gsl_sf_coulomb_wave_sphF_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "CL");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_coulomb_CL_e, NULL, nxp_gsl_sf_coulomb_CL_e, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","coulomb", "CL");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT_REAL, nxp_gsl_sf_coulomb_CL_array, NULL, nxp_gsl_sf_coulomb_CL_array, context, NULL); nxp_identifier_unref(id);
}
