/* -*- c -*- */
#include <gsl/gsl_poly.h>
#include "nxp_kernel.h"
#include "types/nxp_complex.h"
#include "gsl.h"
#include "types/vector.h"


static NxpElement *nxp_gsl_sf_gamma(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_gamma(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lngamma(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lngamma(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *nxp_gsl_sf_gammastar(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_gammastar(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_gammainv(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_gammainv(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_lngamma_complex_e(NxpElementComplex * c, ETC)
{
    gsl_sf_result   lnr,
                    arg;
    NxpElement     *vec;

    gsl_sf_lngamma_complex_e(CXREAL(nxp_element_complex_get_value(c)), CXIMAG(nxp_element_complex_get_value(c)), &lnr,
			     &arg);

    vec = nxp_element_vector_new(2, NULL);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 0) =
	nxp_element_real_new(lnr.val);
    _element_vector_index(NXP_ELEMENT_VECTOR(vec), 1) =
	nxp_element_real_new(arg.val);
    GSL_RETURN(vec);
}

static NxpElement *nxp_gsl_sf_taylorcoeff(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_taylorcoeff(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_fact(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_fact(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_doublefact(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_doublefact(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lnfact(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lnfact(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lndoublefact(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lndoublefact(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_choose(NxpElementInt *arg1 ,NxpElementInt *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_choose(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_ui(NXP_ELEMENT_INT(arg2), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lnchoose(NxpElementInt *arg1 ,NxpElementInt *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lnchoose(nxp_element_int_get_ui(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_ui(NXP_ELEMENT_INT(arg2), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_poch(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_poch(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lnpoch(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lnpoch(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *nxp_gsl_sf_pochrel(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_pochrel(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_gamma_inc_Q(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_gamma_inc_Q(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_gamma_inc_P(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_gamma_inc_P(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_beta(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_beta(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_lnbeta(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_lnbeta(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_beta_inc(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_beta_inc(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}



void
nxp_gsl_sf_gamma_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"gamma");
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "gamma");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_gamma, NULL, nxp_gsl_sf_gamma, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnGamma");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_lngamma, NULL, nxp_gsl_sf_lngamma, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "gammaStar");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_gammastar, NULL, nxp_gsl_sf_gammastar, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "gammaInv");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_gammainv, NULL, nxp_gsl_sf_gammainv, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnGamma");
 nxp_bifunc_define(id, &nxp_prototype__COMPLEX, nxp_gsl_sf_lngamma_complex_e, NULL, nxp_gsl_sf_lngamma_complex_e, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "taylorCoeff");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_taylorcoeff, NULL, nxp_gsl_sf_taylorcoeff, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "factorial");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_fact, NULL, nxp_gsl_sf_fact, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "doubleFactorial");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_doublefact, NULL, nxp_gsl_sf_doublefact, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnFactorial");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_lnfact, NULL, nxp_gsl_sf_lnfact, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnDoubleFactorial");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_lndoublefact, NULL, nxp_gsl_sf_lndoublefact, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "choose");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT, nxp_gsl_sf_choose, NULL, nxp_gsl_sf_choose, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnChoose");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT, nxp_gsl_sf_lnchoose, NULL, nxp_gsl_sf_lnchoose, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "poch");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_poch, NULL, nxp_gsl_sf_poch, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnPoch");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_lnpoch, NULL, nxp_gsl_sf_lnpoch, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "relPoch");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_pochrel, NULL, nxp_gsl_sf_pochrel, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "incQ");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_gamma_inc_Q, NULL, nxp_gsl_sf_gamma_inc_Q, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "incP");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_gamma_inc_P, NULL, nxp_gsl_sf_gamma_inc_P, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "beta");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_beta, NULL, nxp_gsl_sf_beta, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "lnBeta");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_lnbeta, NULL, nxp_gsl_sf_lnbeta, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","gamma", "betaInc");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_sf_beta_inc, NULL, nxp_gsl_sf_beta_inc, context, NULL); nxp_identifier_unref(id);
}
