/* -*- c -*- */
#include <gsl/gsl_sf_legendre.h>
#include "nxp_kernel.h"
#include "gsl.h"


// Legendre Polynomials
static NxpElement *nxp_gsl_sf_legendre_P1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_P1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_P2(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_P2(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_P3(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_P3(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_Pl(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_Pl(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

NxpElement     *
nxp_gsl_sf_legendre_Pl_array(NxpElementInt * LMAX, NxpElementReal * X, ETC)
{
    NxpElement     *vec;
    int             i,
                    size = nxp_element_r_get_value_as_int(LMAX) + 1;
    double         *RESULT_ARRAY = g_new(double, size);
    gsl_sf_legendre_Pl_array(nxp_element_r_get_value_as_int(LMAX), nxp_element_r_get_value_as_double(X), RESULT_ARRAY);
    vec = nxp_element_vector_new(size, NULL);
    for (i = 0; i < size; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(RESULT_ARRAY[i]);
    }
    g_free(RESULT_ARRAY);
    GSL_RETURN(vec);
}

static NxpElement *nxp_gsl_sf_legendre_Q0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_Q0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_Q1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_Q1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_Ql(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_Ql(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}



// Associated Legendre Polynomials and Spherical Harmonics
static NxpElement *nxp_gsl_sf_legendre_Plm(NxpElementInt *arg1 ,NxpElementInt *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_Plm(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg2), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}

NxpElement     *
nxp_gsl_sf_legendre_Plm_array(NxpElementInt * LMAX,
			      NxpElementInt * M, NxpElementReal * X, ETC)
{
    int             i,
                    size = nxp_element_r_get_value_as_int(LMAX) - nxp_element_r_get_value_as_int(M) + 1;
    double         *RESULT_ARRAY;
    NxpElement     *vec;

    RESULT_ARRAY = g_new(double, size);
    gsl_sf_legendre_Plm_array(nxp_element_r_get_value_as_int(LMAX), nxp_element_r_get_value_as_int(M), nxp_element_r_get_value_as_double(X),
			      RESULT_ARRAY);
    vec = nxp_element_vector_new(size, NULL);
    for (i = 0; i < size; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(RESULT_ARRAY[i]);
    }
    g_free(RESULT_ARRAY);
    GSL_RETURN(vec);
}
static NxpElement *nxp_gsl_sf_legendre_sphPlm(NxpElementInt *arg1 ,NxpElementInt *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_sphPlm(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg2), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}

NxpElement     *
nxp_gsl_sf_legendre_sphPlm_array(NxpElementInt * LMAX,
				 NxpElementInt * M,
				 NxpElementReal * X, ETC)
{
    int             i,
                    size = nxp_element_r_get_value_as_int(LMAX) - nxp_element_r_get_value_as_int(M) + 1;
    double         *RESULT_ARRAY;
    NxpElement     *vec;

    RESULT_ARRAY = g_new(double, size);
    gsl_sf_legendre_sphPlm_array(nxp_element_r_get_value_as_int(LMAX), nxp_element_r_get_value_as_int(M), nxp_element_r_get_value_as_double(X),
				 RESULT_ARRAY);
    vec = nxp_element_vector_new(size, NULL);
    for (i = 0; i < size; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(RESULT_ARRAY[i]);
    }
    g_free(RESULT_ARRAY);
    GSL_RETURN(vec);
}

// Conical Functions
static NxpElement *nxp_gsl_sf_conicalP_half(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_half(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_conicalP_mhalf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_mhalf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_conicalP_0(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_conicalP_1(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_conicalP_sph_reg(NxpElementInt *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_sph_reg(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_conicalP_cyl_reg(NxpElementInt *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_conicalP_cyl_reg(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


// Radial Functions for Hyperbolic Space
static NxpElement *nxp_gsl_sf_legendre_H3d_0(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_H3d_0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_H3d_1(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_H3d_1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_legendre_H3d(NxpElementInt *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_legendre_H3d(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}

NxpElement     *
nxp_gsl_sf_legendre_H3d_array(NxpElementInt * LMAX,
			      NxpElementReal * LAMBDA,
			      NxpElementReal * ETA, ETC)
{
    int             i,
                    size = nxp_element_r_get_value_as_int(LMAX) + 1;
    double         *RESULT_ARRAY;
    NxpElement     *vector;

    RESULT_ARRAY = g_new(double, size);
    gsl_sf_legendre_H3d_array(nxp_element_r_get_value_as_int(LMAX), nxp_element_r_get_value_as_double(LAMBDA), nxp_element_r_get_value_as_double(ETA),
			      RESULT_ARRAY);
    vector = nxp_element_vector_new(size, NULL);
    for (i = 0; i < size; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vector), i) =
	    nxp_element_real_new(RESULT_ARRAY[i]);
    }
    g_free(RESULT_ARRAY);
    GSL_RETURN(vector);
}

void
nxp_gsl_sf_legendre_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"legendre");
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "P1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_legendre_P1, NULL, nxp_gsl_sf_legendre_P1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "P2");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_legendre_P2, NULL, nxp_gsl_sf_legendre_P2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "P3");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_legendre_P3, NULL, nxp_gsl_sf_legendre_P3, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "Pl");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_legendre_Pl, NULL, nxp_gsl_sf_legendre_Pl, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "PlVector");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_legendre_Pl_array, NULL, nxp_gsl_sf_legendre_Pl_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "Q0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_legendre_Q0, NULL, nxp_gsl_sf_legendre_Q0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "Q1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_legendre_Q1, NULL, nxp_gsl_sf_legendre_Q1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "Ql");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_legendre_Ql, NULL, nxp_gsl_sf_legendre_Ql, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "Plm");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_legendre_Plm, NULL, nxp_gsl_sf_legendre_Plm, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "PlmVector");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_legendre_Plm_array, NULL, nxp_gsl_sf_legendre_Plm_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "sphPlm");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_legendre_sphPlm, NULL, nxp_gsl_sf_legendre_sphPlm, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "sphPlmVector");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_legendre_sphPlm_array, NULL, nxp_gsl_sf_legendre_sphPlm_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalP_half");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_conicalP_half, NULL, nxp_gsl_sf_conicalP_half, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalP_mhalf");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_conicalP_mhalf, NULL, nxp_gsl_sf_conicalP_mhalf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalP0");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_conicalP_0, NULL, nxp_gsl_sf_conicalP_0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalP1");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_conicalP_1, NULL, nxp_gsl_sf_conicalP_1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalPSphReg");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_REAL, nxp_gsl_sf_conicalP_sph_reg, NULL, nxp_gsl_sf_conicalP_sph_reg, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "conicalPCylReg");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_REAL, nxp_gsl_sf_conicalP_cyl_reg, NULL, nxp_gsl_sf_conicalP_cyl_reg, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "H3d0");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_legendre_H3d_0, NULL, nxp_gsl_sf_legendre_H3d_0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "H3d1");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_legendre_H3d_1, NULL, nxp_gsl_sf_legendre_H3d_1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "H3d");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_REAL, nxp_gsl_sf_legendre_H3d, NULL, nxp_gsl_sf_legendre_H3d, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","legendre", "H3dVector");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_REAL, nxp_gsl_sf_legendre_H3d_array, NULL, nxp_gsl_sf_legendre_H3d_array, context, NULL); nxp_identifier_unref(id);
}
