/* -*- c -*- */
%{
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include "nxp_kernel.h"
#include "element.h"
#include "real.h"
#include "int.h"
#include "nxp_complex.h"
#include "op_bindings.h"
#include "parser.h"
#include "grammar.h"
#include "message.h"
#include "plugins.h"
#include "i18n.h"

#undef YY_INPUT
#define YY_INPUT(b, r, ms) (r = exp_yyinput(b,ms))

%}

 /* start conditions for comments and strings */
%x comment string ident
%option noyywrap

  /* GString used when parsing strings */
  static GString *string_buf = NULL;

  /* Just to check error with separators and components */
  static int separator;


ws	        [ \t]+		
integer		[0-9]+
real		([0-9]*"."[0-9]+)|((([0-9]*"."[0-9]+)|[0-9]+)[Ee][+-]?[0-9]+)
nl 		\n
imag_symbol	(i|j)
size		#

	/* id == one name component of an identifier */
component	([A-Za-z_][0-9A-Za-z_]+)|[:_A-Za-hk-z]
component1	([A-Za-z_][0-9A-Za-z_]*)


  /* This *must* be in sync with what is defined in configure.in */
path_sep	::
path_up         \.\.
range           \.\.\.

%%

[ \t\n]*"else"[ \t\n]*	    {return ELSE; }
"if"			    {return IF; }
[ \t\n]*"then"[ \t\n]*	    {return THEN; }
"while"			    {return WHILE; }
[ \t\n]*"do"[ \t\n]*	    {return DO; }
"for"			    {return FOR; }
"in"			    {return IN; }
{size}			    {return SIZE; }

{imag_symbol}    { 
  explval.atom = nxp_parser_try_parse(NXP_ATOM_CLASS_COMPLEX, "1"); return COMPLEX; }

{real} { explval.atom = nxp_parser_try_parse(NXP_ATOM_CLASS_REAL, exptext); return REAL; }

{integer} 	 { explval.atom = nxp_parser_try_parse(NXP_ATOM_CLASS_INT, exptext);
                           return INT; }

{nl}             { return NEWLINE; }

{ws}*           { ; }

{range}		{ return RANGE; }

\+		{ return OP_PLUS; }
\+\+		{ return OP_INC; }

\=		{ return OP_ATRIB; }

\-		{ return OP_MINUS; }
\-\-		{ return OP_DEC; }

\/		{ return OP_OVER; }

%		{ return OP_PERCENT; }

\*		{ return OP_TIMES; }

\^		{ return OP_CIRC; }

:=		{ return OP_STORE; }

\-\>		{ return OP_SETA; }

=\>		{ return EQ_GT; }

\!		{ return FACTORIAL; }

\<		{ return LT; }

\<=		{ return LE; }

>		{ return GT; }

>=		{ return GE; }

!=		{ return NE; }

==		{ return EQ; }

\/\\		{ return AND; }

\\\/		{ return OR; }

\)		{ return RIGHT_PAR; }

\(		{ return LEFT_PAR; }

\]		{ return RIGHT_RECT; }

\[		{ return LEFT_RECT; }

,		{ return VIRG; }

;		{ return PVIRG; }

\}		{ return RIGHT_CHAV; }

\{		{ return LEFT_CHAV; }

 /* parse C style comments (like this one) -- taken from flex's man page */

"/*"	BEGIN(comment);
<comment>{
    [^*\n]*		/* eat anything that's not a '*' */
    "*"+[^*/\n]*	/* eat up '*'s not followed by '/'s */
    "*"+"/"		BEGIN(INITIAL);
}

{path_sep} {
    BEGIN(ident);
    separator = 1;
    explval.id = nxp_identifier_new();
    nxp_identifier_append_separator(explval.id);
}

({path_up}|{component}) {
    BEGIN(ident);
    explval.id = nxp_identifier_new();
    nxp_identifier_append_component(explval.id, exptext);
    separator = 0;
}

<ident>{path_sep} {
    if (separator == 1)
	nxp_parser_error(N_("Two consecutive separator."));
    else {
	separator = 1;
	nxp_identifier_append_separator(explval.id);
    }
}

<ident>{path_up} {
    if (separator == 1) {
	separator = 0;
	nxp_identifier_append_component(explval.id, exptext);
    }
    else
	nxp_parser_error(N_("Separator needed between two components"));
}

<ident>{component1} {
    if (separator == 1) {
	separator = 0;
	nxp_identifier_append_component(explval.id, exptext);
    }
    else
	nxp_parser_error(N_("Separator needed between two components"));
}

<ident>\[	{
    BEGIN(INITIAL);
    return FUNC;
}

<ident>\(	{
    BEGIN(INITIAL);
    return FUNC1;
}

<ident>(.|\n)	{
    BEGIN(INITIAL);
    yyless(0);
    return IDENTIFIER;
}

 /**** parse strings -- adapted from flex's man page ****/

 /* start of string */
\"	{
    string_buf = g_string_new(NULL);
    BEGIN(string);
}

 /* end of string */
<string>\"	{
    /* saw closing quote - all done */
    BEGIN(INITIAL);
    /* return string constant token type and
     * value to parser
     */
    explval.atom = nxp_parser_try_parse(NXP_ATOM_CLASS_STRING,
				       string_buf->str);
    g_string_free(string_buf, TRUE);
    string_buf = NULL;
    return STRING;
}


<string>\n	{
    nxp_parser_error(N_("unterminated string."));
    /* TODO: abort parser, maybe?... how? */
}


<string>\\[0-7]{1,3} {
    /* octal escape sequence */
    int result;
		   
    (void) sscanf( yytext + 1, "%o", &result );

    if ( result > 0xff )
	nxp_parser_warning(N_("string escape sequence is out-of-bounds."));
    string_buf = g_string_append_c(string_buf, result);
}


<string>\\[0-9]+ {
    nxp_parser_warning(N_("bad escape sequence in string."));
    /* TODO: abort parser, maybe?... how? */
}


<string>{
    \\n  string_buf = g_string_append_c(string_buf, '\n');
    \\t  string_buf = g_string_append_c(string_buf, '\t');
    \\r  string_buf = g_string_append_c(string_buf, '\r');
    \\b  string_buf = g_string_append_c(string_buf, '\b');
    \\f  string_buf = g_string_append_c(string_buf, '\f');
}

<string>\\(.|\n)	string_buf = g_string_append_c(string_buf, yytext[1]);

<string>[^\\\n\"]+	string_buf = g_string_append(string_buf, yytext);

 /**** end of string parsing  *****/


 /* if nothing else matches, return the unmatched single character */
.	{ return exptext[0]; }


%%


